from django.test import TestCase, RequestFactory
from django.contrib.auth.models import AnonymousUser, User

from tracking.models import Event
from tracking.utils import create_event

from .views import UserLogPageView


class CreateEventTests(TestCase):
    """
    Tests for tracking.utils.create_event.
    """

    def setUp(self):
        self.factory = RequestFactory()

    def test_basic_event(self):
        event = create_event('testEvent')
        self.assertEqual(event.name, 'testEvent')

    def test_event_with_request(self):
        request = self.factory.get('/create-event/')
        request.user = AnonymousUser()
        event = create_event('testEvent', request=request)
        self.assertEqual(event.note, '/create-event/')

    def test_event_with_user(self):
        request = self.factory.get('/create-event/')
        user = User.objects.create_user('testuser', email="test@example.com", password='password')
        request.user = user
        event = create_event('testEvent', request=request)
        self.assertEqual(event.user, user)


class UserLogPageViewTests(TestCase):
    """
    Tests for tracking.utils.UserLogPageViewMixin.
    """

    def setUp(self):
        self.factory = RequestFactory()

    def test_page_view_logged(self):
        request = self.factory.get('/log-page-view/')
        user = User.objects.create_user('testuser', email="test@example.com", password='password')
        request.user = user
        response = UserLogPageView.as_view()(request)
        self.assertEqual(Event.objects.count(), 1)
        event = Event.objects.first()
        self.assertEqual(event.name, 'PageViewed')
        self.assertEqual(event.note, '/log-page-view/')
        self.assertEqual(event.user, user)